@@ -27,7 +27,7 @@ class GraphvizTBFormatter(BaseFormatter):
     TABLE_TO_TABLE_FORMAT = '"end${old_table_no}" -> "start${table_no}" [ ltail=cluster_t${old_table_no}, lhead=cluster_t${table_no}, color="#000000", fontcolor="#663333", label="" ]'
     TABLE_TO_TABLE_FORMAT_INVISIBLE = '"end${old_table_no}" -> "start${table_no}" [ ltail=cluster_t${old_table_no}, lhead=cluster_t${table_no}, color ="transparent", label="" ]'
     INSTRUCTION_TO_RESUBMIT_FORMAT = '"l${table_no}_${priority_no}_${entry_no}_${instruction_no}" -> "resubmit${table_no}" [ dirType="none", arrowhead=none, style="dotted", color="#0000CC", fontcolor="#663333", label = "" ]'
-    INSTRUCTION_TO_RESUBMIT_TABLE_FORMAT = '"l${table_no}_${priority_no}_${entry_no}_${instruction_no}" -> "resubmit_to_${resubmit_table_no}" [ dirType="none", arrowhead=none, style="dashed", color="#0000CC", fontcolor="#663333", label = "" ];'
+    INSTRUCTION_TO_RESUBMIT_TABLE_FORMAT = '"l${table_no}_${priority_no}_${entry_no}_${instruction_no}" -> "resubmit_from${table_no}_to${resubmit_table_no}" [ dirType="none", arrowhead=none, style="dashed", color="#0000CC", fontcolor="#663333", label = "" ];'
     
     RESUBMIT_TO_START_FORMAT = '"resubmit${table_no}" -> "start${table_no}" [ style="dotted", color="#0000CC", fontcolor="#663333", label = "" ]'
     RESUBMIT_TO_TABLE_FORMAT = '"l${table_no}_${priority_no}_${entry_no}_${instruction_no}" -> "resubmit_to_t${resubmit_table_no}" [ dirType="none", arrowhead=none, style="dotted", color="#0000CC", fontcolor="#663333", label = "" ]'
@@ -35,7 +35,7 @@ class GraphvizTBFormatter(BaseFormatter):
     START_FORMAT = '"start${table_no}" [ style="rounded", fillcolor="#ddddff", peripheries=1, label="Match START" ]'
     END_FORMAT = '"end${table_no}" [ style="rounded", fillcolor="#ddddff", peripheries=1, label="Match END" ]'
     RESUBMIT_FORMAT = '"resubmit${table_no}" [ style="rounded,dotted", color="#0000cc", fillcolor="#ddddff", peripheries=1, label="Resubmit" ]'
-    RESUBMIT_TABLE_FORMAT = '"resubmit_to_${resubmit_table_no}" [ style="rounded", color="#0000cc", fillcolor="#ddddff", peripheries=1, label="Table ${resubmit_table_no}" ]'
+    RESUBMIT_TABLE_FORMAT = '"resubmit_from${table_no}_to${resubmit_table_no}" [ style="rounded", color="#0000cc", fillcolor="#ddddff", peripheries=1, label="Table ${resubmit_table_no}" ]'
 
     INDENT_IN_GRAPH = '  '
     INDENT_IN_TABLE = '    '
@@ -376,12 +376,12 @@ node [
                 self.resubmit_edges.append("\n" + self.INDENT_IN_TABLE \
                     + self._format_instruction_to_resubmit(table_no, priority_no, entry_no, instruction_no))
                 self.resubmit_loop = True
-            elif self.util.d_push(re.match(r'.* resubmit\(\d*,(\d+)\)', actions)) is not None:
+            elif self.util.d_push(re.match(r'.* resubmit\(\d*,(\d+)\)', actions)) is not None or self.util.d_push(re.match(r'.* goto_table:(\d+)', actions)) is not None:
                 match = self.util.d_pop()
                 resubmit_table_no = match.group(1)
                 formatted_str += "\n" + self.INDENT_IN_PRIORITY_BLOCK + \
                     self._format_instruction_resubmit(table_no, priority_no, entry_no, instruction_no, actions)
-                self.resubmit_tables.append(resubmit_table_no)
+                self.resubmit_tables.append([table_no, resubmit_table_no])
                 self.resubmit_edges.append("\n" + self.INDENT_IN_TABLE \
                     + self._format_instruction_to_resubmit_table(\
                         table_no, priority_no, entry_no, instruction_no, resubmit_table_no))
@@ -443,8 +443,8 @@ node [
     def _format_priority_to_end(self, table_no, priority_no, entry_no):
         return self.INDENT_IN_TABLE + self.priority_to_end_template.substitute({"table_no":table_no, "priority_no":priority_no, "entry_no":entry_no, "instruction_no":"0"})
 
-    def _format_resubmit_table(self, resubmit_table_no):
-        return self.INDENT_IN_TABLE + self.resubmit_table_template.substitute({"resubmit_table_no":resubmit_table_no})
+    def _format_resubmit_table(self, table_no, resubmit_table_no):
+        return self.INDENT_IN_TABLE + self.resubmit_table_template.substitute({"table_no":table_no, "resubmit_table_no":resubmit_table_no})
 
     def _format_resubmit_loop(self, table_no):
         if self.output_resubmit_loop:
@@ -464,11 +464,18 @@ node [
 
     def _format_resubmit_tables(self):
         format_resubmit_tables = []
+        resubmit_table_sets = []
+
         if len(self.resubmit_tables) == 0:
             return ""
-        for resubmit_table in list(set(self.resubmit_tables)):
+#        for table_no, resubmit_table in list(set(self.resubmit_tables)):
+        for resubmit_table_set in self.resubmit_tables:
+            if not resubmit_table_set in resubmit_table_sets:
+                resubmit_table_sets.append(resubmit_table_set)
+
+        for table_no, resubmit_table in resubmit_table_sets:
             if resubmit_table is not None:
-                format_resubmit_tables.append(self._format_resubmit_table(resubmit_table))
+                format_resubmit_tables.append(self._format_resubmit_table(table_no, resubmit_table))
         return "\n".join(format_resubmit_tables)
 
     def _format_table_to_table(self, table_no, old_table_no):
@@ -508,7 +515,7 @@ node [
             self.util.debug_out("i_no:  " + str(instruction_no) + ": " + inst)
             instruction_set[instruction_no] += "<f" + str(
                 f_no) + "> " + inst + "|"
-            if inst.startswith("resubmit") or inst.startswith("output:") or inst.startswith("LOCAL") or inst.startswith("CONTROLLER:"):
+            if inst.startswith("resubmit") or inst.startswith("goto_table:") or inst.startswith("output:") or inst.startswith("LOCAL") or inst.startswith("CONTROLLER:"):
                 if action_no < (len(instructions) - 1):
                     instruction_no += 1
                     instruction_set.append("")
